# -*- coding: utf-8 -*-
# @Time    : 2023/7/10 下午9:40
# @Author  : sudoskys
# @File    : event.py
# @Software: PyCharm

def help_message():
    return """
    /help - HELP YOURSELF
    /chat - Chat with me :)
    /task - Function enable
    /ask - Chat with func_disable, 禁止函数
    /tool - 工具列表
    /clear - 删除自己的记录
    /auth - POWER
    
Private Chat Only:
    /bind - RSS
    /unbind - RSS
    /set_endpoint - <apikey>#<endpoint>
    /clear_endpoint - Clear endpoint and key
    /env - 配置变量,use as shell
    /token - bind your token
    /token_clear - clear your token
    /func_ban - ban function
    /func_unban - unban function

!Please confirm that that bot instance is secure, some plugins may be dangerous on unsafe instance.
"""
